// Global constants
var MAX_TEMP_PARAM_CNT = 16;

/** 
 @brief Creates new ID required for storing to PropertyTree for room modes
 
 @param -
  
 @return New ID value
*/
function getNewIdValue(sNode) {
    var idValue = 0;
    var baseNode = Property.getNode(sNode);
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    if (oArrayNode != null) {
      for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
          var oChildNode = oArrayNode[iIndex];
          if (oChildNode.getChild('id') != null) {
              if (oChildNode.getChild('id').getValue() != null)
                  if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                      idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
          }
      }
    }
    return idValue;
}

/*
function storeSubParameter(root, path, value) {
  Property.setProperty(root +'/' + path, value);
  Property.setFlag(root + '/' + path, 'ARCHIVE', true);
  lInfo.logln("STORING " + root + "/" + path + " -> " + value);
}
*/

function getDefaultPlayerPropertyID(nRoomID) {
   var idValue = -1;	
   var baseNode = Property.getNode('upnp-players');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
         if (oChildNode.getChild('roomID') != null) {
           if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
             if (oChildNode.getChild('id').getValue() != null) {
               idValue = oChildNode.getChild('id').getValue()
               break;
             }
           }
         }
     }
   }
   return idValue;
}

function getDefaultActivityID(nActivityID) {
   var idValue = -1;
   var baseNode = Property.getNode('upnp-players');
   if (baseNode != null) {
     var oArrayNode = baseNode.getChildren();
     for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
       var oChildNode = oArrayNode[iIndex];
       if ((oChildNode.getChild('id') != null)
           && (oChildNode.getChild('id').getValue() != null)
           && (oChildNode.getChild('id').getValue() == nActivityID)) {
         idValue = oChildNode.getChild('id').getValue()
         break;
       }
     }
   }
   return idValue;
}

function storePlayer(sValue) {
    try {
        var oObject = JSON.parse(unescape(sValue));
                     
        var roomID = parseInt(oObject.roomID, 10);
        var playerPropertyID = -1;
        if (oObject.id!=null)
        	playerPropertyID=oObject.id;
        if (playerPropertyID < 0)   
        {
          playerPropertyID = getNewIdValue('upnp-players');
        }
        
        lInfo.logln("Save player : "+ playerPropertyID);
          
        // storing data to propertyTree
        storeSubParameter('upnp-players', playerPropertyID+'/id', playerPropertyID);
        storeSubParameter('upnp-players', playerPropertyID+'/roomID', unescape(oObject.roomID));
        storeSubParameter('upnp-players', playerPropertyID+'/name', unescape(oObject.name));
        storeSubParameter('upnp-players', playerPropertyID+'/ipAddress', unescape(oObject.ipAddress));
        storeSubParameter('upnp-players', playerPropertyID+'/port', unescape(oObject.port));
        storeSubParameter('upnp-players', playerPropertyID+'/typeID', unescape(oObject.typeID));
        Property.getNode('upnp-players/' + playerPropertyID).removeChild('togglePauseButton');
        Property.getNode('upnp-players/' + playerPropertyID).removeChild('dimmUpButton');
        Property.getNode('upnp-players/' + playerPropertyID).removeChild('dimmDownButton');
        if (oObject.togglePauseButton!=null)
            if (oObject.togglePauseButton!='')
            	storeSubParameter('upnp-players', playerPropertyID+'/togglePauseButton', (oObject.togglePauseButton));  
        if (oObject.dimmUpButton!=null)
            if (oObject.dimmUpButton!='')
            	storeSubParameter('upnp-players', playerPropertyID+'/dimmUpButton', (oObject.dimmUpButton));        
        if (oObject.dimmDownButton!=null)
            if (oObject.dimmDownButton!='')
            	storeSubParameter('upnp-players', playerPropertyID+'/dimmDownButton', (oObject.dimmDownButton));        
        Property.store();
    } 
    catch (e) {
        lWarn.logln("Error by saving storePlayer: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
}

function storeActivity(sValue) {
    try {
        var oObject = JSON.parse(unescape(sValue));
                     
        var activityID = parseInt(oObject.id, 10);
        var actPropertyID = getDefaultActivityID(activityID);
        
        if (actPropertyID < 0)   
        {
          actPropertyID = getNewIdValue('activity');
        }
        
        lInfo.logln("Save activity : "+ actPropertyID);
          
        // storing data to propertyTree
        storeSubParameter('activity', actPropertyID+'/id', activityID);
        storeSubParameter('activity', actPropertyID+'/useDefaultLoudness', oObject.bSetDefaultLoudness);
        storeSubParameter('activity', actPropertyID+'/defaultLoudness', oObject.nLoudness);
        storeSubParameter('activity', actPropertyID+'/stopWhenAway', oObject.bStopAway);
        storeSubParameter('activity', actPropertyID+'/doorbellPlayTime', oObject.nDoorbellTime);
        storeSubParameter('activity', actPropertyID+'/stopOnUndo', oObject.bStopUndo);

        if (oObject.presetValues != null) {
          for (var iIndex=0;iIndex<oObject.presetValues.length;iIndex++) {
            var sPresetIndex = oObject.presetValues[iIndex].id;
            var sPresetValue = oObject.presetValues[iIndex].value;

            var sMediaValue = "";
            var mediaInfo = Property.getNode('tmpMediaInfo/valuePreset'+iIndex);
            if (mediaInfo != null) {              
              if (sPresetValue != "")            
                sMediaValue = mediaInfo.getValue();
              storeSubParameter('activity', actPropertyID+'/valuePresetMedia'+sPresetIndex, sMediaValue);
            }

            storeSubParameter('activity', actPropertyID+'/valuePreset'+sPresetIndex, sPresetValue);
            storeSubParameter('activity', actPropertyID+'/valuePresetSceneID'+sPresetIndex, oObject.presetValues[iIndex].sceneID);
          }
        }
        
        if (oObject.activityValues != null) {
          for (var iIndex=0;iIndex<oObject.activityValues.length;iIndex++) {
            var sActivityIndex = oObject.activityValues[iIndex].id;
            var sActivityValue = oObject.activityValues[iIndex].value;
            
            var sMediaValue = "";
            var mediaInfo = Property.getNode('tmpMediaInfo/valueActivity'+iIndex);
            if (mediaInfo != null) {               
              if (sActivityValue != "")           
                sMediaValue = mediaInfo.getValue();
              storeSubParameter('activity', actPropertyID+'/valueActivityMedia'+sActivityIndex, sMediaValue);
            }
            
            storeSubParameter('activity', actPropertyID+'/valueActivity'+sActivityIndex, sActivityValue);
            storeSubParameter('activity', actPropertyID+'/valueActivitySceneID'+sActivityIndex, oObject.activityValues[iIndex].sceneID);
          }
        }
        Property.store();
        
        var baseNode = Property.getNode('tmpMediaInfo');
        if (baseNode != null) {
            var oArrayNode = baseNode.getChildren();
            for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                var oChildNode = oArrayNode[iIndex];
                baseNode.removeChild(oChildNode);
            }
        }

        lInfo.logln("Save activity END!");
    } 
    catch (e) {
        lWarn.logln("Error by saving activityPlayer: " + e);
        for ( var i in e) {
            lWarn.logln(i + ': ' + e[i]);
        }
    }
}



if (raisedEvent.name === 'running') {
    lInfo.logln("running players.js");
    Property.load();
	checkRaumfeld();
}
else
if (raisedEvent.name == 'upnp-controller.savegrid') {
 lInfo.logln("savegrid "+raisedEvent.parameter.actions);
  if (raisedEvent.parameter.actions == 'save') {   
    storePlayer(raisedEvent.parameter.value);
    var evt = new Event("upnp-controller-saved", {});
    evt.raise();
  }
  else
    if (raisedEvent.parameter.actions == 'delete') { 
        lInfo.logln("Start delete");
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
                
        if (typeof(oIDObject)=='number')
            oIDObject=[oIDObject];
        
        for (var sKey in oIDObject) {
         var baseNode = Property.getNode("upnp-players");
         if (baseNode != null)
        lInfo.logln("Delete child" + oIDObject[sKey]);
             baseNode.removeChild(oIDObject[sKey]);
        }
        
        Property.store();

        var evt = new Event("upnp-controller-saved",  {});
        evt.raise();
    }
    else
      if (raisedEvent.parameter.actions == 'saveActivity') { 
        storeActivity(raisedEvent.parameter.value);
      }
      if (raisedEvent.parameter.actions == 'clearMediaInfo') { 
        lInfo.logln("Clear media");
        var baseNode = Property.getNode('tmpMediaInfo');
        if (baseNode != null) {
            var oArrayNode = baseNode.getChildren();
            for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                var oChildNode = oArrayNode[iIndex];
                baseNode.removeChild(oChildNode);
            }
        }
      }
}
else
  if (raisedEvent.name == 'upnp-controller.getdata') 
  {
        if (raisedEvent.parameter.actions == 'fetchPlayList') { 
          var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
          lInfo.logln('fetchPlayList');
                                          
          var s1 = executeUpnpCmd(UPNP_GET_MEDIA_URL_POSITION, 
                                  oObject.typeID,
                                  oObject.ipAddress,
                                  oObject.port,
                                  0, /* new value, needed only for some cmd */
                                  '', /* playlist url, needed only for some cmd */
                                  oObject.id /* player id */);
          var sReturnValue = s1.data;
          
          if (oObject.typeID == 'RAUMFELD') {
            if (s1.mediaInfo.data.indexOf('dlna-playcontainer')>-1) {
              sReturnValue = s1.mediaInfo.data;  
            }
            
            if (!oObject.fetchPlayList && s1.track > 0) {
              sReturnValue = "track:" + s1.track + "*@*" + sReturnValue;
            }
            lInfo.logln('sReturnValue='+sReturnValue);  
            if (oObject.nodeID != "") {
              // needed only for radio stations to be showed corretly inside Raumfeld app 
              storeSubParameter('tmpMediaInfo', oObject.nodeID, sReturnValue);
              var sXmlValue = "";
              if (s1.mediaInfo.dataXML != "" && s1.mediaInfo.dataXML.indexOf("audioBroadcast.radio")>-1) 
              {  
                sXmlValue = s1.mediaInfo.dataXML;
                sReturnValue = s1.mediaInfo.data;
              }
              storeSubParameter('tmpMediaInfo', oObject.nodeID, sXmlValue);
            }
          }
         
          var evt = new Event("upnp-controller-result", {returnValue : sReturnValue});
          evt.raise();
        }
}

