/**
 * @file main.js
 * 
 * @brief This file contains implementation of User interface for the UPNP
 *        controller app.
 * 
 * @author: mobile2DS
 * 
 * @date: 03.08.2013
 */

var fLoading = false;
var oTimedEvent;

var oTasterDevices = [];

Ext.define('DSS.addon.UpnpController', {
	extend : 'DSS.addon.Framework',
	fExpertMode : false,
	config : {
		appName : 'UPNP Controller App',
		appId : 'upnp-controller-app',
		appVersion : '0.1',
		appIcon : 'default_icon.png'
	},

	getHelp : function() {
		return Ext.create('Ext.Component', {
			loader : {
				url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
				autoLoad : true,
				disableCaching : false
			}
		});
	},

	upnp_renderers : null,

	temporarayDisableControls : true,

	arrayPresetStates : null,
	arrayActivities : null,

	pnlLoudness : null,
	pnlStop : null,
	pnlDoorbell : null,
	pnlStopUndo : null,

	comboPlayers : null,
	defLoudness : null,
	defLoudnessValue : null,
	stopAway : null,
	defDorbellValue : null,
	stopUndo : null,
	leftPanel : null,
	rightPanel : null,
	pnlPresetRow : null,
	pnlActRow : null,
	saveActivityData : null,

	arrayPresetsElements : null,
	arrayActivitiesElements : null,
	actionPropertyValues : null,

	storeData : null,

	setEmptyValues : function() {
		var me = this;

		me.defLoudness.setValue(false);
		me.defLoudnessValue.setValue(0);
		me.stopAway.setValue(false);
		me.defDorbellValue.setValue(0);
		me.stopUndo.setValue(false);

		for ( var i = 0; i < me.arrayPresetsElements.length; i++) {
			me.arrayPresetsElements[i].edit.setValue("");
		}

		for ( var i = 0; i < me.arrayActivitiesElements.length; i++) {
			me.arrayActivitiesElements[i].edit.setValue("");
		}
	},

	loadActions : function(nActionID) {
		var me = this;

		if (nActionID >= 0) {
			for ( var i = 0; i < me.arrayPresetsElements.length; i++) {
				me.arrayPresetsElements[i].edit.enable();
				me.arrayPresetsElements[i].btn.enable();
			}

			for ( var i = 0; i < me.arrayActivitiesElements.length; i++) {
				me.arrayActivitiesElements[i].edit.enable();
				me.arrayActivitiesElements[i].btn.enable();
			}
            
            dss.raiseEvent('upnp-controller.savegrid', {
                actions : 'clearMediaInfo',
                value : null
            });

			me.pnlLoudness.enable();
			me.pnlStop.enable();
			me.pnlDoorbell.enable();
			me.pnlStopUndo.enable();

			me.saveActivityData.enable();

			sString = dss.ajaxSyncRequest('/json/property/query', {
				'query' : '/scripts/upnp-controller/activity/*(*)/*(*)/*(*)/*(*)'
			});
			activitiesValues = Ext.JSON.decode(sString);

			// load player actions on selected player ID
			me.setEmptyValues();
            actionPropertyValues = activitiesValues.result.activity;
            
			if (actionPropertyValues != null) {
				for ( var iIndex = 0; iIndex < actionPropertyValues.length; iIndex++) {
					if (actionPropertyValues[iIndex] != null && actionPropertyValues[iIndex].id == nActionID) {
						if (actionPropertyValues[iIndex].useDefaultLoudness != null)
							me.defLoudness.setValue(actionPropertyValues[iIndex].useDefaultLoudness);
						if (actionPropertyValues[iIndex].defaultLoudness != null)
							me.defLoudnessValue.setValue(actionPropertyValues[iIndex].defaultLoudness);
						if (actionPropertyValues[iIndex].stopWhenAway != null)
							me.stopAway.setValue(actionPropertyValues[iIndex].stopWhenAway);
						if (actionPropertyValues[iIndex].doorbellPlayTime != null)
							me.defDorbellValue.setValue(actionPropertyValues[iIndex].doorbellPlayTime);
						if (actionPropertyValues[iIndex].stopOnUndo != null)
							me.stopUndo.setValue(actionPropertyValues[iIndex].stopOnUndo);

						for ( var i = 0; i < me.arrayPresetsElements.length; i++) {
							me.arrayPresetsElements[i].edit.setValue(actionPropertyValues[iIndex]["valuePreset" + me.arrayPresetsElements[i].id].replace(/&amp;/g, '&'));
						}

						for ( var i = 0; i < me.arrayActivitiesElements.length; i++) {
							me.arrayActivitiesElements[i].edit.setValue(actionPropertyValues[iIndex]["valueActivity" + me.arrayActivitiesElements[i].id].replace(/&amp;/g, '&'));
						}

					}
				}
			}
		}
	},

	saveActions : function() {
		var me = this;

		nSelectedPlayer = me.comboPlayers.getValue();
		if (nSelectedPlayer >= 0) {

			var aP = new Array();
			for ( var iIndex = 0; iIndex < me.arrayPresetsElements.length; iIndex++) {
				aP.push({
					'id' : me.arrayPresetsElements[iIndex].id,
					'sceneID' : me.arrayPresetsElements[iIndex].sceneID,
					'value' : me.arrayPresetsElements[iIndex].edit.rawValue,
				});
			}
			var aA = new Array();
			for ( var iIndex = 0; iIndex < me.arrayActivitiesElements.length; iIndex++) {
				aA.push({
					'id' : me.arrayActivitiesElements[iIndex].id,
					'sceneID' : me.arrayActivitiesElements[iIndex].sceneID,
					'value' : me.arrayActivitiesElements[iIndex].edit.rawValue,
				});                                                            
			}

			var storeData = new Object({
				'id' : nSelectedPlayer,
				'bSetDefaultLoudness' : me.defLoudness.getValue(),
				'nLoudness' : me.defLoudnessValue.getValue(),
				'bStopAway' : me.stopAway.getValue(),
				'nDoorbellTime' : me.defDorbellValue.getValue(),
				'bStopUndo' : me.stopUndo.getValue(),
				'presetValues' : aP,
				'activityValues' : aA
			});
			dss.raiseEvent('upnp-controller.savegrid', {
				actions : 'saveActivity',
				value : Ext.JSON.encode(storeData)
			});
		}
	},

	getContent : function() {

		oTasterDevices = [{name:'not selected', dsid:''}];
		dss.staticDataModel.initCaches();
		for ( var iIndex = 0; iIndex < dss.staticDataModel.deviceCache.length; iIndex++) {
			if (dss.staticDataModel.deviceCache[iIndex].buttonActiveGroup == 8)
				if (dss.staticDataModel.deviceCache[iIndex].tasterMode == 15)
					if (dss.staticDataModel.deviceCache[iIndex].binaryInputs.length == 0) {
						oTasterDevices.push({
							name: dss.staticDataModel.deviceCache[iIndex].name + ' (' + dss.staticDataModel.deviceCache[iIndex].dSID + ')',
							dsid: dss.staticDataModel.deviceCache[iIndex].dSID});
					}
		}

		var me = this;
		me.arrayPresetStates = new Array({
			'id' : 1,
			'sceneID' : 5
		}, {
			'id' : 2,
			'sceneID' : 17
		}, {
			'id' : 3,
			'sceneID' : 18
		}, {
			'id' : 4,
			'sceneID' : 19
		}, {
			'id' : 11,
			'sceneID' : 33
		}, {
			'id' : 12,
			'sceneID' : 20
		}, {
			'id' : 13,
			'sceneID' : 21
		}, {
			'id' : 14,
			'sceneID' : 22
		}, {
			'id' : 20,
			'sceneID' : 2
		}, {
			'id' : 21,
			'sceneID' : 7
		}, {
			'id' : 22,
			'sceneID' : 23
		}, {
			'id' : 23,
			'sceneID' : 24
		}, {
			'id' : 24,
			'sceneID' : 25
		});

		me.arrayActivities = new Array({
			'id' : 1,
			'name' : 'Door Bell',
			'sceneID' : 73
		}, {
			'id' : 2,
			'name' : 'Coming Home',
			'sceneID' : 71
		}, {
			'id' : 3,
			'name' : 'Fire',
			'sceneID' : 76
		}, {
			'id' : 4,
			'name' : 'Panic',
			'sceneID' : 65
		}, {
			'id' : 5,
			'name' : 'Alarm 1',
			'sceneID' : 74
		}, {
			'id' : 6,
			'name' : 'Alarm 2',
			'sceneID' : 83
		}, {
			'id' : 7,
			'name' : 'Sleep',
			'sceneID' : 69
		}, {
			'id' : 8,
			'name' : 'Waking up',
			'sceneID' : 70
		});
		/*
		 * var arrayTypes = new Array({"id":"SONOS", "name":"SONOS"},
		 * {"id":"NP1100", "name":"NP1100"}, {"id":"SAMSUNGTV",
		 * "name":"SAMSUNGTV"}, {"id":"BUBBLE", "name":"BUBBLE"});
		 */
		var arrayTypes = new Array({
			"id" : "SONOS",
			"name" : "SONOS"
		}, {
			"id" : "NP1100",
			"name" : "NP1100"
		}, {
			"id" : "BUBBLE",
			"name" : "BUBBLE"
		}, {
            "id" : "RAUMFELD",
            "name" : "RAUMFELD"
        });

        
		var arrayRooms = new Array();
		if (dss.staticDataModel.nameCache == null)
			dss.staticDataModel.initCaches()
		if (dss.staticDataModel.nameCache != null) {
			for ( var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
				var oZone = dss.staticDataModel.nameCache[iIndex];
				if (oZone.ZoneID > 0) {
					arrayRooms.push({
						"roomID" : oZone.ZoneID,
						"roomName" : oZone.name
					});
				}
			}
		}

        
		me.upnp_renderers = Ext.create('Ext.data.Store', {
			fields : [ 'id', 'name' ],
			data : arrayTypes
		});

		var oConfig = {
			defaultSortCol : 'name',
			disabledProperty : 'disabled',
			storeConfig : {
				storeId : 'pushStore',
				fields : [ {
					name : 'id'
				}, // {name:'disabled', sortType:'asBoolean'},
				{
					name : 'name',
					sortType : 'asUCText'
				}, {
					name : 'roomID',
					sortType : 'asUCText'
				}, {
					name : 'roomName',
					sortType : 'asUCText'
				}, {
					name : 'ipAddress',
					sortType : 'asUCText'
				}, {
					name : 'port',
					sortType : 'asUCText'
				}, {
					name : 'typeID',
					sortType : 'asUCText'
				},	{
					name : 'togglePauseButton',
					sortType : 'asUCText'
				}, {
					name : 'dimmUpButton',
					sortType : 'asUCText'
				}, {
					name : 'dimmDownButton',
					sortType : 'asUCText'
				}]
			},
			multiselect : true,
			columns : [
			/*
			 * { header : _('Active'), dataIndex : 'disabled', resizable :
			 * false, xtype: 'disabledcolumn', width : 50 },
			 */{
				header : _('Player name'),
				dataIndex : 'name',
				xtype : 'textcolumn',
				width : 200
			}, {
				header : _('Room'),
				dataIndex : 'roomName',
				xtype : 'textcolumn',
				flex : 1
			}, {
				header : _('IP Address'),
				dataIndex : 'ipAddress',
				xtype : 'textcolumn',
				flex : 1
			}, {
				header : _('Port'),
				dataIndex : 'port',
				xtype : 'textcolumn',
				flex : 1
			}, {
				header : _('Device type'),
				dataIndex : 'typeID',
				xtype : 'textcolumn',
				flex : 1
			} ],
			customButtonsRAlign : [ {
				text : _('Requery Devices from UPNP'),
				iconCls : '',
				handler : function(grid, entry) {
					dss.raiseEvent('upnp-controller.generic', {
						actions : 'refreshDeviceList'
					});

				}
			} ],
			buttonText : {
				newButton : _('New player'),
				editButton : _('Edit player'),
				deleteButton : _('Delete player')
			},
			editorConfig : {
				title : _('Player info'),
				staticControls : [ {
					type : 'text',
					label : _('Player name'),
					id : 'name'
				}, /*
					 * { type : 'text', label : _('RoomID'), id : 'roomID' },
					 */
				{
					type : 'combobox',
					label : _('Room'),
					id : 'roomID',
					data : arrayRooms,
					storevalueid : 'roomID',
					storedisplayid : 'roomName',
				}, {
					type : 'text',
					label : _('IP Address'),
					id : 'ipAddress'
				}, {
					type : 'text',
					label : _('Port'),
					id : 'port'
				}, {
					type : 'combobox',
					label : _('Device type'),
					id : 'typeID',
					data : arrayTypes,
					storevalueid : 'id',
					storedisplayid : 'name',
				}, {
					type : 'hidden',
					label : _('id'),
					id : 'id'
				}, {
					type : 'combobox',
					label : _('toggle pause'),
					id : 'togglePauseButton',
					data : oTasterDevices,
					storevalueid : 'dsid',
					storedisplayid : 'name',
				}, {
					type : 'combobox',
					label : _('Dimm up'),
					id : 'dimmUpButton',
					data : oTasterDevices,
					storevalueid : 'dsid',
					storedisplayid : 'name',
				}, {
					type : 'combobox',
					label : _('Dimm down'),
					id : 'dimmDownButton',
					data : oTasterDevices,
					storevalueid : 'dsid',
					storedisplayid : 'name',
				} ],
			},
			listeners : {
				'loadDataStore' : function(grid) {
					me.loadParameter(grid);
				},
				'saveEntry' : function(grid, values) {
					dss.raiseEvent('upnp-controller.savegrid', {
						actions : 'save',
						value : Ext.JSON.encode(values)
					})
					me.waitAndReload(grid);
				},
				'deleteEntry' : function(grid, entry) {
					var idVal;
					if (entry.get != null) {
						idVal = entry.get('id');
					} else {
						idVal = [];
						for ( var iIndex = 0; iIndex < entry.length; iIndex++)
							if (entry[iIndex] != null)
								idVal.push(entry[iIndex].get('id'));
					}
					dss.raiseEvent('upnp-controller.savegrid', {
						actions : 'delete',
						id : Ext.JSON.encode(idVal)
					})
					me.waitAndReload(grid);
				}
			}
		};

		var oMainGrid = Ext.create('DSS.gridPanel', oConfig);
		oPlayers = Ext.create('Ext.panel.Panel', {
			title : _('Players'),
			layout : 'fit',
			items : [ oMainGrid ]
		});

		var sString = dss.ajaxSyncRequest('/json/property/query', {
			'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
		});
		var result = Ext.JSON.decode(sString);

		oResultObject = dss.parsePropertyQueryAnswer(result, [ {
			name : 'id',
			type : 'simple'
		}, {
			name : 'name',
			type : 'simple'
		} ]);
		for ( var i = 0; i < oResultObject.length; i++) {
			oResultObject[i].name = unescape(oResultObject[i].name);
		}

		me.storeData = Ext.create('Ext.data.Store', {
			fields : [ "id", "name" ],
		});
		me.storeData.add(oResultObject);

		/*
		 * sString = dss.ajaxSyncRequest('/json/property/query', { 'query' :
		 * '/scripts/upnp-controller/activity/*(*)/*(*)/*(*)/*(*)' }); result =
		 * Ext.JSON.decode(sString);
		 * 
		 * actionPropertyValues=dss.parsePropertyQueryAnswer(result,[{name:'id',type:'simple'},
		 * {name:'useDeafaultLoudness',type:'simple'},
		 * {name:'defaultLoudness',type:'simple'}, {name:'stopWhenAway',
		 * type:'simple'}, {name:'dorbellPlayTime',type:'simple'},
		 * {name:'stopOnUndo',type:'simple'}]);
		 */
		me.comboPlayers = Ext.create('Ext.form.field.ComboBox', {
			store : me.storeData,
			queryMode : 'local',
			displayField : 'name',
			valueField : 'id',
			fieldLabel : _('Select Player'),
			width : 300,
			listeners : {
				'select' : function(objCombo) {
					var nSelectedModeID = objCombo.getValue();
					me.loadActions(nSelectedModeID);
				}
			}
		});

		/* LOUDNESS */
		me.defLoudness = Ext.create('Ext.form.field.Checkbox', {
			disabledCls : 'disabledRow',
			fieldLabel : "",
			align : 'middle',
		})

		me.defLoudnessCheck = Ext.create('Ext.form.Label', {
			text : _("Set default loudness"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'defLoudnessCheck'
		});

		me.defLoudnessValue = Ext.create('Ext.form.field.Number', {
			allowBlank : false,
			allowDecimals : false,
			margin : '0 0 0 5',
			width : 52,
			value : 0,
			minValue : 0,
			maxValue : 100,
			minText : _("The minimum value is 0"),
			maxText : _("The maximum value is 100")

		});

		me.defLoudnessPerc = Ext.create('Ext.form.Label', {
			text : _("%"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'defLoudnessPerc'
		});

		me.pnlLoudness = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ me.defLoudness, me.defLoudnessCheck, me.defLoudnessValue, me.defLoudnessPerc ],
			layout : 'column',
			margin : '10 0 0 0',
			width : 350
		});
		/** ****************************************************** */

		/* Stop music when leving home */
		me.stopAway = Ext.create('Ext.form.field.Checkbox', {
			disabledCls : 'disabledRow',
			fieldLabel : "",
			align : 'middle',
		})

		me.stopCheck = Ext.create('Ext.form.Label', {
			text : _("Stop music when leaving home"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'lblCheckBox3'
		});

		me.pnlStop = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ me.stopAway, me.stopCheck ],
			layout : 'column',
			margin : '10 0 0 0',
			width : 350
		});
		/** ****************************************************** */

		/* Table header */
		var tableTop1 = Ext.create('Ext.form.Label', {
			text : _("Presets"),
			margin : '0 0 0 5',
			cls : 'tableTop',
			id : 'tableTop1',
			width : 100,
			background : 'gray'
		});

		var tableTop2 = Ext.create('Ext.form.Label', {
			text : _("What to play"),
			margin : '0 0 0 5',
			cls : 'tableTop',
			id : 'tableTop2',
			width : 200,
		});

		me.pnlTableTop = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ tableTop1, tableTop2 ],
			layout : 'column',
			cls : 'tableTop',
			margin : '10 0 0 0',
			width : 350
		});
		/** ****************************************** */

		me.arrayPresetsElements = new Array();

		var arrayPresetPnls = new Array();
		for ( var iIndex = 0; iIndex < me.arrayPresetStates.length; iIndex++) {
			// presets table
			var aState = me.arrayPresetStates[iIndex];

			var tableTopItem = Ext.create('Ext.form.Label', {
				text : _("Preset " + aState.id),
				margin : '0 0 0 5',
				id : 'presetCell' + aState.id,
				width : 100,
			});

			var tableTopItemEdit = Ext.create('Ext.form.field.Text', {
				margin : '0 0 0 0',
				id : 'presetCellEdit' + aState.id,
				itemId : 'editPreset' + aState.id,
			});

			var tableTopItemButton = Ext.create('Ext.button.Button', {
				text : _('...'),
                margin : '0 0 0 0',
				itemId : (iIndex + 1),
				listeners : {
					click : function() {
						var nSelectedPlayerID = me.comboPlayers.getValue();
						if (nSelectedPlayerID >= 0) {
							var sString = dss.ajaxSyncRequest('/json/property/query', {
								'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
							});
							var result = Ext.JSON.decode(sString);

							oResultObject = dss.parsePropertyQueryAnswer(result, [ {
								name : 'id',
								type : 'simple'
							}, {
								name : 'ipAddress',
								type : 'simple'
							}, {
								name : 'port',
								type : 'simple'
							}, {
								name : 'typeID',
								type : 'simple'
							} ]);
							var sendData = null;
							for ( var i = 0; i < oResultObject.length; i++) {
								if (oResultObject[i].id == nSelectedPlayerID)
									sendData = oResultObject[i];
							}

							if (sendData != null) {
								var nIndex = this.getItemId() - 1;
                                sendData.nodeID = "valuePreset" + nIndex;
                                sendData.fetchPlayList = false;

								me.arrayPresetsElements[nIndex].edit.setValue(_("Fetching data..."));
								dss.raiseEvent('upnp-controller.getdata', {
									actions : 'fetchPlayList',
									value : Ext.JSON.encode(new Object(sendData))
								});

								var iSubscriptID = Math.floor(Math.random() * 100000000000);
								dss.ajaxSyncRequest('/json/event/subscribe', {
									subscriptionID : iSubscriptID,
									name : 'upnp-controller-result'
								});
								dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
									subscriptionID : iSubscriptID,
									timeout : 2 * 1000
								}, function(data) {
									var val = JSON.parse(data);
                                    //var val = JSON.parse(unescape(data));
									var editValue = "";
									if (val != null && val.result.events[0] != null && val.result.events[0].properties != null) {
										editValue = val.result.events[0].properties.returnValue;
                                    }
									me.arrayPresetsElements[nIndex].edit.setValue(editValue);

									dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
										subscriptionID : iSubscriptID,
										name : 'upnp-controller-result'
									});
								});
							}
						}
					}
				}
			});
            var tableTopItemButtonPL = Ext.create('Ext.button.Button', {
                text : _('List'),
                margin : '0 0 0 1',
                itemId : (iIndex + 101),
                listeners : {
                    click : function() {
                        var nSelectedPlayerID = me.comboPlayers.getValue();
                        if (nSelectedPlayerID >= 0) {
                            var sString = dss.ajaxSyncRequest('/json/property/query', {
                                'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
                            });
                            var result = Ext.JSON.decode(sString);

                            oResultObject = dss.parsePropertyQueryAnswer(result, [ {
                                name : 'id',
                                type : 'simple'
                            }, {
                                name : 'ipAddress',
                                type : 'simple'
                            }, {
                                name : 'port',
                                type : 'simple'
                            }, {
                                name : 'typeID',
                                type : 'simple'
                            } ]);
                            var sendData = null;
                            for ( var i = 0; i < oResultObject.length; i++) {
                                if (oResultObject[i].id == nSelectedPlayerID)
                                    sendData = oResultObject[i];
                            }

                            if (sendData != null) {
                                var nIndex = this.getItemId() - 101;
                                sendData.nodeID = "valuePreset" + nIndex;
                                sendData.fetchPlayList = true;

                                me.arrayPresetsElements[nIndex].edit.setValue(_("Fetching data..."));
                                dss.raiseEvent('upnp-controller.getdata', {
                                    actions : 'fetchPlayList',
                                    value : Ext.JSON.encode(new Object(sendData))
                                });

                                var iSubscriptID = Math.floor(Math.random() * 100000000000);
                                dss.ajaxSyncRequest('/json/event/subscribe', {
                                    subscriptionID : iSubscriptID,
                                    name : 'upnp-controller-result'
                                });
                                dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
                                    subscriptionID : iSubscriptID,
                                    timeout : 2 * 1000
                                }, function(data) {
                                    var val = JSON.parse(data);
                                    //var val = JSON.parse(unescape(data));
                                    var editValue = "";
                                    if (val != null && val.result.events[0] != null && val.result.events[0].properties != null) {
                                        editValue = val.result.events[0].properties.returnValue;
                                    }
                                    me.arrayPresetsElements[nIndex].edit.setValue(editValue);

                                    dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                                        subscriptionID : iSubscriptID,
                                        name : 'upnp-controller-result'
                                    });
                                });
                            }
                        }
                    }
                }
            });
            
			tableTopItemEdit.disable();
			tableTopItemButton.disable();
			me.arrayPresetsElements.push(new Object({
				'edit' : tableTopItemEdit,
				'btn' : tableTopItemButton,
				'id' : aState.id,
				'sceneID' : aState.sceneID
			}));

			me.pnlPresetRow = Ext.create('Ext.container.Container', {
				defaultType : 'numberfield',
				items : [ tableTopItem, tableTopItemEdit, tableTopItemButton, tableTopItemButtonPL ],
				layout : 'column',
				margin : '10 0 0 0',
				width : 350
			});

			arrayPresetPnls.push(me.pnlPresetRow);
			// me.pnlPresetRow.disable();
		}

		arrayPresetPnls.unshift(me.comboPlayers, me.pnlLoudness, me.pnlStop, me.pnlTableTop);

		var leftPanelData = Ext.create('Ext.panel.Panel', {
			layout : {
				type : 'vbox',
				align : 'left'
			},
			border : 0,
			width : 350,
			height : 1100,
			enabled : false,
			items : arrayPresetPnls
		});

		me.leftPanel = Ext.create('Ext.panel.Panel', {
			layout : {
				type : 'vbox',
				align : 'center'
			},
			border : 0,
			width : 350,
			enabled : false,
			items : [ leftPanelData ]
		});

		/* Play music */
		var defDorbell = Ext.create('Ext.form.Label', {
			text : _("Play doorbell for"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'lbldefDorbell'
		});

		me.defDorbellValue = Ext.create('Ext.form.field.Number', {
			allowBlank : false,
			allowDecimals : false,
			margin : '0 0 0 5',
			width : 52,
			value : 0,
			minValue : 1,
			maxValue : 60,
			minText : _("The minimum value is 1"),
			maxText : _("The maximum value is 10")

		});

		var defDorbellSec = Ext.create('Ext.form.Label', {
			text : _("sec. and restart music afterwards"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'lbldefDorbellSec'
		});

		me.pnlDoorbell = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ defDorbell, me.defDorbellValue, defDorbellSec ],
			layout : 'column',
			margin : '37 0 0 0',
			width : 350
		});
		/** ****************************************************** */

		/* Stop music on undo fire.. */
		me.stopUndo = Ext.create('Ext.form.field.Checkbox', {
			disabledCls : 'disabledRow',
			fieldLabel : "",
			align : 'middle',
		})

		var stopUndoCheck = Ext.create('Ext.form.Label', {
			text : _("Restart music when reseting fire, panic and alarms"),
			margin : '0 0 0 5',
			cls : 'x-form-item-label',
			id : 'lblstopCheck'
		});

		me.pnlStopUndo = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ me.stopUndo, stopUndoCheck ],
			layout : 'column',
			margin : '10 0 0 0',
			width : 350
		});
		/** ****************************************************** */

		/* Table header */
		var tableTopR1 = Ext.create('Ext.form.Label', {
			text : _("Activities"),
			margin : '0 0 0 5',
			cls : 'tableTop',
			id : 'tableTopR1',
			width : 100,
			background : 'gray'
		});

		var tableTopR2 = Ext.create('Ext.form.Label', {
			text : _("What to play"),
			margin : '0 0 0 5',
			cls : 'tableTop',
			id : 'tableTopR2',
			width : 200,
		});

		var pnlTableTop2 = Ext.create('Ext.container.Container', {
			defaultType : 'numberfield',
			items : [ tableTopR1, tableTopR2 ],
			layout : 'column',
			cls : 'tableTop',
			margin : '10 0 0 0',
			width : 350
		});
		/** ****************************************** */

		var arrayActivitiesPnls = new Array();
		me.arrayActivitiesElements = new Array();

		for ( var iIndex = 0; iIndex < me.arrayActivities.length; iIndex++) {
			var nID = me.arrayActivities[iIndex].id;
			var sName = me.arrayActivities[iIndex].name;
			var tableTopItemAct = Ext.create('Ext.form.Label', {
				text : _(sName),
				margin : '0 0 0 5',
				id : 'activityCell' + nID,
				width : 100,
			});

			var tableTopItemActEdit = Ext.create('Ext.form.field.Text', {
				margin : '0 0 0 5',
				id : 'activityCellEdit' + nID,
				itemId : 'editAct' + nID,
			});
			var tableTopItemActButton = Ext.create('Ext.button.Button', {
				text : _('...'),
				itemId : nID,
				listeners : {
					click : function() {
						var nSelectedPlayerID = me.comboPlayers.getValue();
						if (nSelectedPlayerID >= 0) {
							var sString = dss.ajaxSyncRequest('/json/property/query', {
								'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
							});
							var result = Ext.JSON.decode(sString);

							oResultObject = dss.parsePropertyQueryAnswer(result, [ {
								name : 'id',
								type : 'simple'
							}, {
								name : 'ipAddress',
								type : 'simple'
							}, {
								name : 'port',
								type : 'simple'
							}, {
								name : 'typeID',
								type : 'simple'
							} ]);
							var sendData = null;
							for ( var i = 0; i < oResultObject.length; i++) {
								if (oResultObject[i].id == nSelectedPlayerID)
									sendData = oResultObject[i];
							}

							if (sendData != null) {
								var nIndex = this.getItemId() - 1;
                                sendData.nodeID = "valueActivity" + nIndex;
                                
								me.arrayActivitiesElements[nIndex].edit.setValue(_("Fetching data..."));
								dss.raiseEvent('upnp-controller.getdata', {
									actions : 'fetchPlayList',
									value : Ext.JSON.encode(new Object(sendData))
								});

								var iSubscriptID = Math.floor(Math.random() * 100000000000);
								dss.ajaxSyncRequest('/json/event/subscribe', {
									subscriptionID : iSubscriptID,
									name : 'upnp-controller-result'
								});
								dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
									subscriptionID : iSubscriptID,
									timeout : 2 * 1000
								}, function(data) {
									var val = JSON.parse(data);
                                    //var val = JSON.parse(unescape(data));
									var editValue = "";
                                    if (val != null && val.result.events[0] != null && val.result.events[0].properties != null) {
                                        editValue = val.result.events[0].properties.returnValue;
                                    }
									me.arrayActivitiesElements[nIndex].edit.setValue(editValue);

									dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
										subscriptionID : iSubscriptID,
										name : 'upnp-controller-result'
									});
								});
							}
						}
					}
				}
			});

			tableTopItemActEdit.disable();
			tableTopItemActButton.disable();
			me.arrayActivitiesElements.push(new Object({
				'edit' : tableTopItemActEdit,
				'btn' : tableTopItemActButton,
				'id' : nID,
				'sceneID' : me.arrayActivities[iIndex].sceneID
			}));

			me.pnlActRow = Ext.create('Ext.container.Container', {
				defaultType : 'numberfield',
				items : [ tableTopItemAct, tableTopItemActEdit, tableTopItemActButton ],
				layout : 'column',
				margin : '10 0 0 0',
				width : 350
			});
			arrayActivitiesPnls.push(me.pnlActRow);
		}
		arrayActivitiesPnls.unshift(me.pnlDoorbell, me.pnlStopUndo, pnlTableTop2);

		me.saveActivityData = Ext.create('Ext.button.Button', {
			text : _('Save Changes'),
			margin : '10 0 0 0',
			handler : function() {
				me.saveActions();
			}
		});
		me.saveActivityData.disable();
		arrayActivitiesPnls.push(me.saveActivityData);

		me.rightPanel = Ext.create('Ext.panel.Panel', {
			layout : {
				type : 'vbox',
				align : 'center'
			},
			border : 0,
			width : 350,
			margin : '0 0 0 40',
			enabled : false,
			items : arrayActivitiesPnls
		});

		var oActivities = Ext.create('Ext.panel.Panel', {
			layout : {
				type : 'hbox'
			},
			title : _('Activities'),
			border : 0,
			margin : '10 10 10 10',
			items : [ me.leftPanel, me.rightPanel ]
		});

		me.pnlLoudness.disable();
		me.pnlStop.disable();
		me.pnlDoorbell.disable();
		me.pnlStopUndo.disable();

		return Ext.create('Ext.tab.Panel', {
			layout : 'fit',
			cls : 'generalTab',
			items : [ oPlayers, oActivities ]
		});
	},

	reloadPlayersCombo : function() {
		var me = this;
		var sString = dss.ajaxSyncRequest('/json/property/query', {
			'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
		});
		var result = Ext.JSON.decode(sString);

		oResultObject = dss.parsePropertyQueryAnswer(result, [ {
			name : 'id',
			type : 'simple'
		}, {
			name : 'name',
			type : 'simple'
		} ]);
		for ( var i = 0; i < oResultObject.length; i++) {
			oResultObject[i].name = unescape(oResultObject[i].name);
		}
		me.storeData.removeAll();
		me.storeData.add(oResultObject);
	},

	loadParameter : function(grid) {
		grid.setStatus(_('Loading ...'));
		var sString = dss.ajaxSyncRequest('/json/property/query', {
			'query' : '/scripts/upnp-controller/upnp-players/*(*)/*(*)/*(*)/*(*)'
		});
		var result = Ext.JSON.decode(sString);

		oResultObject = dss.parsePropertyQueryAnswer(result, [ {
			name : 'id',
			type : 'simple'
		}, {
			name : 'name',
			type : 'simple'
		}, {
			name : 'roomID',
			type : 'simple'
		}, {
			name : 'ipAddress',
			type : 'simple'
		}, {
			name : 'port',
			type : 'simple'
		}, {
			name : 'typeID',
			type : 'simple'
		}, {
			name : 'togglePauseButton',
			type : 'simple'
		}, {
			name : 'dimmUpButton',
			type : 'simple'
		}, {
			name : 'dimmDownButton',
			type : 'simple'
		} ]);
		if (oResultObject != null) {
			for ( var iIndex = 0; iIndex < oResultObject.length; iIndex++) {
				if (oResultObject[iIndex].togglePauseButton==null)
					oResultObject[iIndex].togglePauseButton='';
				if (oResultObject[iIndex].dimmUpButton==null)
					oResultObject[iIndex].dimmUpButton='';
				if (oResultObject[iIndex].dimmDownButton==null)
					oResultObject[iIndex].dimmDownButton='';
				oResultObject[iIndex].grouping = unescape(oResultObject[iIndex].name).split(' ')[0];
				oResultObject[iIndex].roomName = dss.staticDataModel.getNameForZone(oResultObject[iIndex].roomID);
			}
			grid.loadParameter(oResultObject);
		}
		grid.setStatus(_('Ready'));
	},
	waitAndReload : function(grid) {
		grid.setStatus(_('Saving ...'));
		var me = this;
		var iSubscriptID = Math.floor(Math.random() * 100000000000);
		dss.ajaxSyncRequest('/json/event/subscribe', {
			subscriptionID : iSubscriptID,
			name : 'upnp-controller-saved'
		});
		dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
			subscriptionID : iSubscriptID,
			timeout : 60 * 1000
		}, function() {
			me.loadParameter(grid);
			me.reloadPlayersCombo();
			dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
				subscriptionID : iSubscriptID,
				name : 'upnp-controller-saved'
			});
		});
	}

});

Ext.onReady(function() {
	dss.buildUpLang([ 'locale/{languageSuffix}/upnp-controller.po' ]);
	var upnpControllerApp = Ext.create('DSS.addon.UpnpController');
	upnpControllerApp.initPage();
});
